import { 
  select, 
  json, 
  tree, 
  hierarchy, 
  linkHorizontal, 
  zoom,
  event
} from 'd3';

const svg = select('svg');
const width = document.body.clientWidth;
const height = document.body.clientHeight;

const margin = { top: 0, right: 50, bottom: 0, left: 75};
const innerWidth = width - margin.left - margin.right;
const innerHeight = height - margin.top - margin.bottom;
// Create tree layout with d3.tree
const treeLayout = tree().size([innerHeight, innerWidth]);	// this(tree()) sets d.y & d.x values of each node for node positions/creating links

const zoomG = svg
		.attr('width', width)
		.attr('height', height)
  .append('g')

const g = zoomG.append('g')
    .attr('transform', `translate(${margin.left}, ${margin.top})`);

svg.call(zoom().on('zoom', () => {
  zoomG.attr('transform', event.transform);
}));

json('data.json')
	.then(data => {
		const root = hierarchy(data);  // parse data as hierarchical stuctured data
  	const links = treeLayout(root).links();	// returns array of objects that we can use for linkages between nodes
    const linkPathGenerator = linkHorizontal()
      .x(d => d.y)
      .y(d => d.x);
  
  g.selectAll('path').data(links)
    .enter().append('path')
      .attr('d', linkPathGenerator);
  
  g.selectAll('text').data(root.descendants())
    .enter().append('text')
      .attr('x', d => d.y)
      .attr('y', d => d.x)
      .attr('dy', '0.32em')	// secret ingredient for centering text to line(edge)
      .attr('text-anchor', d => d.children ? 'middle' : 'start')
      .attr('font-size', d => 3.25 - d.depth + 'em') // value for d.depth is assigned automatically when hierarchy(data) is called
      .text(d => d.data.data.id);
});